package tech.nevets.dliteserver;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;

public class Connection implements Runnable {
    private Socket s;
    private InputStream is;
    private OutputStream os;
    private MessageDistributer md;
    private String username;

    public Connection(Socket s, MessageDistributer md) throws IOException {
        this.s = s;
        this.is = s.getInputStream();
        this.os = s.getOutputStream();
        this.md = md;
        String buf = "";
        if (is.read() != 0x1) throw new IOException("Unexpected Command");
        for (int i = is.read(); i != 0xf && i != -1; i = is.read()) {
            buf += (char) i;
        }
        username = buf.replace("\r", "").replace("\n", "").trim();
    }

    @Override
    public void run() {
        md.registerStream(this);
        byte[] buf = new byte[1024];

        try {
            loop: while (!s.isClosed()) {
                byte cmd = (byte) is.read();
                int i = 0;
                for (byte b = (byte) is.read(); b != 0xf && i < 1023; b = (byte) is.read()) {
                    buf[i] = b;
                    i++;
                }

                switch (cmd) {
                    case 0x1 -> md.sendUserConnect(new String(Arrays.copyOfRange(buf, 0, i))); // User Connect
                    case 0x2 -> {
                        MessageDistributer.closeUserConnection(this);
                        md.sendUserDisconnect(new String(Arrays.copyOfRange(buf, 0, i))); // User Disconnect
                        break loop;
                    }
                    case 0x3 -> md.sendMessage(username, new String(Arrays.copyOfRange(buf, 0, i))); // Send Message
                }
            }
            if (!s.isClosed()) {
                try {
                    s.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        } catch (IOException e) {
            System.out.println("Connection disconnected unexpectedly");
            MessageDistributer.closeUserConnection(this);
        }
    }

    public String getUsername() {
        return username;
    }

    public OutputStream getOutputStream() {
        return this.os;
    }

    public void updateUsername(String username) {
        this.username = username;
    }

    public void sendOnline(String onlineList) {
        try {
            os.write(0x4);
            os.write(onlineList.getBytes(StandardCharsets.UTF_8));
            os.write(0xf);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
