package tech.nevets.dliteserver;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.util.*;

/*
Commands:
 - 0x1: User connect
 - 0x2: User disconnect
 - 0x3: Message
 - 0x4: Online List

 - 0xff: End of PDU

 Usable: 00 - 1f, 80 - ff
 */

public class MessageDistributer extends OutputStream {
    private static List<Connection> userStreams = new ArrayList<>();
    private static MessageDistributer instance;
    private byte[] buf = new byte[1024];
    private short bufPointer = 0;

    public MessageDistributer() {
        instance = this;
    }

    @Override
    public void write(int b) {
        buf[bufPointer] = ((byte) b);
        bufPointer++;
    }

    public void write(byte[] b, int off, int len) throws IOException {
        for (int i = 0 ; i < len; i++) {
            write(b[off + i]);
        }
        flush();
    }

    @Override
    public void flush() {
        for (Connection conn : userStreams) {
            try {
                System.out.println("Sending to: " + conn.getUsername());
                conn.getOutputStream().write(Arrays.copyOfRange(buf, 0, bufPointer));
                conn.getOutputStream().flush();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        bufPointer = 0;
    }

    public void sendUserConnect(String user) {
        try {
            this.write(0x1);
            this.write((user + "\r\n").getBytes(StandardCharsets.UTF_8));
            this.write(0xf);
            this.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void sendUserDisconnect(String user) {
        try {
            this.write(0x2);
            this.write((user + "\r\n").getBytes(StandardCharsets.UTF_8));
            this.write(0xf);
            this.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void sendMessage(String user, String message) {
        try {
            this.write(0x3);
            this.write((user + "> " + message).getBytes(StandardCharsets.UTF_8));
            this.write(0xf);
            this.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void registerStream(Connection conn) {
        int i = 0;
        String username;
        for (username = conn.getUsername(); existingUsername(username); username = conn.getUsername() + "." + i) {
            i++;
        }
        conn.updateUsername(username);
        sendUserConnect(username);
        userStreams.add(conn);
        conn.sendOnline(getOnlineList());
        sendMessage("SERVER", "Welcome " + username + "!\r\n");
    }

    private String getOnlineList() {
        StringBuilder sb = new StringBuilder();
        for (Connection c : userStreams) {
            sb.append(c.getUsername()).append("\n");
        }
        return sb.toString();
    }

    private boolean existingUsername(String username) {
        for (Connection c : userStreams) {
            if (c.getUsername().equals(username)) return true;
        }
        return false;
    }

    private static MessageDistributer getInstance() {
        return instance;
    }

    public static void closeUserConnection(Connection conn) {
        userStreams.remove(conn);
        getInstance().sendUserDisconnect(conn.getUsername());
        getInstance().sendMessage("SERVER", ("Goodbye, " + conn.getUsername() + "!\r\n"));
    }
}
